Struct truck_platform::Light[][src]

pub struct Light {
    pub position: Point3,
    pub color: Vector3,
    pub light_type: LightType,
}

Light

There is no limit to the number of lights that can be added to a Scene. The information about the lights is sent to the shader as a storage buffer (cf: Scene::lights_buffer()).

Fields

position: Point3

position of light

color: Vector3

[0, 1] range RGB color of light

light_type: LightType

type of light source: point or uniform

Implementations

impl Light[src]

pub fn buffer(&self, device: &Device) -> BufferHandler[src]

Creates a UNIFORM buffer of light.

This method is provided only for the advanced developer utility, and not used by Scene.

Shader Example

layout(// binding info //) uniform Light {
    vec4 position;      // the position of light, position.w == 1.0
    vec4 color;         // the color of light, color.w == 1.0
    uvec4 light_type;   // Point => uvec4(0, 0, 0, 0), Uniform => uvec4(1, 0, 0, 0)
};