Struct truck_platform::SceneDescriptor [−][src]
Configures of Scene.
Fields
background: Colorbackground color. Default is Color::BLACK.
camera: Cameracamera of the scene. Default is Camera::default().
lights: Vec<Light>All lights in the scene. Default is vec![Light::default()].
sample_count: u32sample count for anti-aliasing by MSAA. 1, 2, 4, 8, or 16.
Implementations
impl SceneDescriptor[src]
pub fn camera_buffer(&self, handler: &DeviceHandler) -> BufferHandler[src]
Creates a UNIFORM buffer of camera.
The bind group provides Scene holds this uniform buffer.
Shader Example
layout(set = 0, binding = 0) uniform Camera {
mat4 camera_matrix; // the camera matrix
mat4 camera_projection; // the projection into the normalized view volume
};
pub fn lights_buffer(&self, device: &Device) -> BufferHandler[src]
Creates a STORAGE buffer of all lights.
The bind group provides Scene holds this uniform buffer.
Shader Example
struct Light {
vec4 position; // the position of light, position.w == 1.0
vec4 color; // the color of light, color.w == 1.0
uvec4 light_type; // Point => uvec4(0, 0, 0, 0), Uniform => uvec4(1, 0, 0, 0)
};
layout(set = 0, binding = 1) buffer Lights {
Light lights[];
};