WindowScene

Struct WindowScene 

pub struct WindowScene { /* private fields */ }
Utility for wrapper

Implementations

Source

impl WindowScene

pub async fn from_window( window: Arc<Window>, scene_desc: &WindowSceneDescriptor, ) -> Self

Initialize scene compatible with window.

pub const fn window(&self) -> &Arc<Window>

Get the reference of initializing window.

pub const fn surface(&self) -> &Arc<Surface<'_>>

Get the reference of surface.

pub fn size_alignment(&mut self)

Adjusts the size of the backend buffers (depth or sampling buffer) to the size of the window.

pub fn render_frame(&mut self)

Render scene to initializing window.

Methods from Deref<Target = Scene>

pub fn compatible_texture(&self) -> Texture

Creates compatible texture for render attachment.

Remarks

The usage of texture is TextureUsages::RENDER_ATTACHMENT | TetureUsages::COPY_SRC.

pub fn device_handler(&self) -> &DeviceHandler

Returns the reference of its own DeviceHandler.

pub fn device(&self) -> &Arc<Device>

Returns the reference of the device.

pub fn queue(&self) -> &Arc<Queue>

Returns the reference of the queue.

pub fn elapsed(&self) -> Duration

Returns the elapsed time since the scene was created.

pub fn descriptor(&self) -> &SceneDescriptor

Returns the reference of the descriptor.

pub fn descriptor_mut(&mut self) -> SceneDescriptorMut<'_>

Returns the mutable reference of the descriptor.

Remarks

When the return value is dropped, the depth buffer and sampling buffer are automatically updated. Use studio_config_mut if you only want to update the colors of the camera, lights, and background.

pub fn studio_config(&self) -> &StudioConfig

Returns the reference of the studio configuration.

pub fn studio_config_mut(&mut self) -> &mut StudioConfig

Returns the mutable reference of the studio configuration.

pub fn bind_group_layout(&self) -> &BindGroupLayout

Returns the bind group layout in the scene.

pub fn camera_buffer(&self) -> BufferHandler

Creates a UNIFORM buffer of the camera.

The bind group provides Scene holds this uniform buffer.

Shader Example
layout(set = 0, binding = 0) uniform Camera {
    mat4 camera_matrix;     // the camera matrix
    mat4 camera_projection; // the projection into the normalized view volume
};
pub fn lights_buffer(&self) -> BufferHandler

Creates a STORAGE buffer of all lights.

The bind group provides Scene holds this uniform buffer.

Shader Example
struct Light {
    vec4 position;      // the position of light, position.w == 1.0
    vec4 color;         // the color of light, color.w == 1.0
    uvec4 light_type;   // Point => uvec4(0, 0, 0, 0), Uniform => uvec4(1, 0, 0, 0)
};

layout(set = 0, binding = 1) buffer Lights {
    Light lights[]; // the number of lights must be gotten from another place
};
pub fn scene_status_buffer(&self) -> BufferHandler

Creates a UNIFORM buffer of the scene status.

The bind group provides Scene holds this uniform buffer.

Shader Example
layout(set = 0, binding = 2) uniform Scene {
    vec4 bk_color;  // color of back ground
    float time;     // elapsed time since the scene was created.
    uint nlights;   // the number of lights
};
pub fn scene_bind_group(&self) -> BindGroup

Creates bind group.

Shader Examples

Suppose binded as set = 0.

layout(set = 0, binding = 0) uniform Camera {
    mat4 camera_matrix;     // the camera matrix
    mat4 camera_projection; // the projection into the normalized view volume
};

struct Light {
    vec4 position;      // the position of light, position.w == 1.0
    vec4 color;         // the color of light, color.w == 1.0
    uvec4 light_type;   // Point => uvec4(0, 0, 0, 0), Uniform => uvec4(1, 0, 0, 0)
};

layout(set = 0, binding = 1) buffer Lights {
    Light lights[];
};

layout(set = 0, binding = 2) uniform Scene {
    float time;     // elapsed time since the scene was created.
    uint nlights;   // the number of lights
};
pub fn add_object<R: Rendered>(&mut self, object: &R) -> bool

Adds a render object to the scene.

If there already exists a render object with the same ID, replaces the render object and returns false.

pub fn set_visibility<R: Rendered>(&mut self, object: &R, visible: bool) -> bool

Sets the visibility of a render object.

If there does not exist the render object in the scene, does nothing and returns false.

pub fn add_objects<'a, R, I>(&mut self, objects: I) -> bool
where R: 'a + Rendered, I: IntoIterator<Item = &'a R>,

Adds render objects to the scene.

If there already exists a render object with the same ID, replaces the render object and returns false.

pub fn remove_object<R: Rendered>(&mut self, object: &R) -> bool

Removes a render object from the scene.

If there does not exist the render object in the scene, does nothing and returns false.

pub fn remove_objects<'a, R, I>(&mut self, objects: I) -> bool
where R: 'a + Rendered, I: IntoIterator<Item = &'a R>,

Removes render objects from the scene.

If there exists a render object which does not exist in the scene, returns false.

pub fn clear_objects(&mut self)

Removes all render objects from the scene.

pub fn number_of_objects(&self) -> usize

Returns the number of the render objects in the scene.

pub fn update_vertex_buffer<R: Rendered>(&mut self, object: &R) -> bool

Synchronizes the information of vertices of object in the CPU memory and that in the GPU memory.

If there does not exist the render object in the scene, does nothing and returns false.

pub fn update_vertex_buffers<'a, R, I>(&mut self, objects: I) -> bool
where R: 'a + Rendered, I: IntoIterator<Item = &'a R>,

Synchronizes the information of vertices of objects in the CPU memory and that in the GPU memory.

If there exists a render object which does not exist in the scene, returns false.

pub fn update_bind_group<R: Rendered>(&mut self, object: &R) -> bool

Synchronizes the information of bind group of object in the CPU memory and that in the GPU memory.

If there does not exist the render object in the scene, does nothing and returns false.

pub fn update_bind_groups<'a, R, I>(&mut self, objects: I) -> bool
where R: 'a + Rendered, I: IntoIterator<Item = &'a R>,

Synchronizes the information of bind group of object in the CPU memory and that in the GPU memory.

If there exists a render object which does not exist in the scene, returns false.

pub fn update_pipeline<R: Rendered>(&mut self, object: &R) -> bool

Synchronizes the information of pipeline of object in the CPU memory and that in the GPU memory.

If there does not exist the render object in the scene, does nothing and returns false.

pub fn update_pipelines<'a, R, I>(&mut self, objects: I) -> bool
where R: 'a + Rendered, I: IntoIterator<Item = &'a R>,

Synchronizes the information of pipeline of object in the CPU memory and that in the GPU memory.

If there exists a render object which does not exist in the scene, returns false.

pub fn render(&self, view: &TextureView)

Renders the scene to view.

pub async fn render_to_buffer(&self) -> Vec<u8>

Render image to buffer.